#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-06-22
#
# rewrite of the original CMK check checkpoint_ha_problem
# 2021-09-08: rewritten for CMK 2.0
#             changed service name from 'HA Problem' to HA Critical Devices' --> see Check Pont CLI reference guide
#             for 'cphaprobe -l list', ClusterXL monitoring commands, viewing critical devices.
# https://sc1.checkpoint.com/documents/R81.10/WebAdminGuides/EN/CP_R81.10_CLI_ReferenceGuide/Topics-CLIG/CXLG/Viewing-Critical-Devices.htm
#
# has only one item "HA Problems"
# critical if service missing,
# warning if new service is found
# critical for service not ok
#
# ToDo: wato for level of new/missing service
# ToDo: wato for ignore HA device
#

from dataclasses import dataclass
from typing import Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointHaProblem:
    dev_status: str
    description: str


def parse_checkpoint_ha_problems(string_table: StringTable) -> Dict[str, CheckpointHaProblem]:
    cp_ha_device = {}
    for name, dev_status, description in string_table:
        cp_ha_device[name] = CheckpointHaProblem(
            dev_status=dev_status,
            description=description
        )
    return cp_ha_device


def discovery_checkpoint_ha_problems(section: Dict[str, CheckpointHaProblem]) -> DiscoveryResult:
    yield Service(parameters={'cp_ha_device': list(section.keys())})


def check_checkpoint_ha_problems(params, section: Dict[str, CheckpointHaProblem]) -> CheckResult:
    cp_ha_device = params['cp_ha_device']

    summary = f'# of Critical Devices: {len(section):d}/{len(cp_ha_device):d} (found/expected)'
    if len(cp_ha_device) == len(section):
        yield Result(state=State.OK, summary=summary)
    else:
        yield Result(state=State.WARN, notice=summary)

    for key in section.keys():
        text = f'Device name: {key}'
        if section[key].description != '':
            text += f', Description: {section[key].description}'
        text += f', Status: {section[key].dev_status}'
        if section[key].dev_status == 'OK':
            yield Result(state=State.OK, notice=text)
        else:
            yield Result(state=State.CRIT, notice=text)
        if key not in cp_ha_device:
            yield Result(state=State.WARN, notice=f'New critical device found: {key}')

    for name in cp_ha_device:
        if name not in section.keys():
            yield Result(state=State.CRIT, notice=f'Missing critical device: {name}')


register.snmp_section(
    name='checkpoint_ha_problems',
    parse_function=parse_checkpoint_ha_problems,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.5.13.1',  # CHECKPOINT-MIB::
        oids=[
            '2',  # haProblemName
            '3',  # haProblemStatus
            '6',  # haProblemDescr
        ]),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_ha_problems',
    service_name='HA Critical Devices',
    discovery_function=discovery_checkpoint_ha_problems,
    check_function=check_checkpoint_ha_problems,
    check_default_parameters={
    },
    check_ruleset_name='checkpoint_ha_problems',
)
